'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var EndPoint = function () {
    function EndPoint(name, _ref) {
        var _this = this;

        var url = _ref.url;
        var defaultRequestConfig = _ref.defaultRequestConfig;

        _classCallCheck(this, EndPoint);

        this.name = name;
        this.url = url;
        this.defaultRequestConfig = defaultRequestConfig || {};

        // Construct the methods for http methods that don't need data
        ['delete', 'get', 'head', 'options'].forEach(function (method) {
            _this[method] = function (args) {
                var config = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
                return function (dispatch) {
                    return _this.request(dispatch, method, args, config);
                };
            };
        });

        // Construct the methods for http methods that do need data
        ['post', 'put', 'patch'].forEach(function (method) {
            _this[method] = function (args, data) {
                var config = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
                return function (dispatch) {
                    return _this.request(dispatch, method, args, config, data);
                };
            };
        });
    }

    _createClass(EndPoint, [{
        key: 'transformUrl',
        value: function transformUrl() {
            var args = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

            var url = this.url;
            // Replace all :arg with the actual value
            Object.keys(args).forEach(function (key) {
                var argRe = new RegExp(':' + key + '(?=[^\w])');
                url = url.replace(argRe, args[key]);
            });
            // Remove remaining arguments not present in args
            url = url.replace(/:[\w]+(?=[^\w])/, '');
            // Remove double slashes
            url = url.replace(/\/+/g, '/');
            return url;
        }
    }, {
        key: 'actionType',
        value: function actionType(action) {
            return '@@super-api@' + this.name + '_' + action;
        }
    }, {
        key: 'actionSuccess',
        value: function actionSuccess(response) {
            return {
                type: this.actionType('success'),
                data: response.data,
                status: response.status
            };
        }
    }, {
        key: 'actionError',
        value: function actionError(error) {
            if (error.response) {
                // Server responded with an error
                return {
                    type: this.actionType('error'),
                    error: error.response.data,
                    status: error.response.status
                };
            } else {
                // Request was malformed
                return {
                    type: this.actionType('error'),
                    error: error.message
                };
            }
        }
    }, {
        key: 'actionRequest',
        value: function actionRequest(method) {
            return {
                type: this.actionType('request'),
                method: method
            };
        }
    }, {
        key: 'actionReset',
        value: function actionReset() {
            return {
                type: this.actionType('reset')
            };
        }
    }, {
        key: 'reset',
        value: function reset() {
            var _this2 = this;

            return function (dispatch) {
                return dispatch(_this2.actionReset());
            };
        }
    }, {
        key: 'request',
        value: function request(dispatch, method, args, config) {
            var _this3 = this;

            var data = arguments.length <= 4 || arguments[4] === undefined ? undefined : arguments[4];

            dispatch(this.actionRequest(method));
            return _axios2.default.request(_extends({
                url: this.transformUrl(args),
                method: method,
                data: data
            }, this.defaultRequestConfig[method], config)).then(function (response) {
                return dispatch(_this3.actionSuccess(response));
            }).catch(function (error) {
                return dispatch(_this3.actionError(error));
            });
        }
    }, {
        key: 'reduce',
        value: function reduce(state, action) {
            var defaultState = {
                loading: false,
                sync: false,
                syncing: false,
                data: {},
                error: null
            };

            if (typeof state === 'undefined') {
                return defaultState;
            }

            switch (action.type) {
                case this.actionType('request'):
                    return _extends({}, state, {
                        loading: true,
                        sync: false,
                        syncing: true,
                        data: {},
                        error: null
                    });
                case this.actionType('success'):
                    return _extends({}, state, {
                        loading: false,
                        sync: true,
                        syncing: false,
                        data: action.data,
                        error: null
                    });
                case this.actionType('error'):
                    return _extends({}, state, {
                        loading: false,
                        sync: false,
                        syncing: false,
                        data: {},
                        error: action.error
                    });
                case this.actionType('reset'):
                    return defaultState;
                default:
                    return state;
            }
        }
    }]);

    return EndPoint;
}();

exports.default = EndPoint;